\name{jk.means}
\alias{jk.means}
\title{Compute univariate statistics.}
\usage{jk.means(connection, input.table, analyse, group.by=list(),
    where="TRUE", mu0=0, alpha.notch=0.025, return.format="df",
    debug=0, secret="")}
\description{Compute univariate statistics.}
\details{\code{jk.means} computes moments- and rank-based non-robust and robust univariate statistics
and returns approximate results including a small confidentiality protection error, 
together with approximate information about this error.

Moments-based statistics reported:

\tabular{ll}{
n       \tab no. of non-null values \cr
sum     \tab sum \cr
uss     \tab square sum \cr
us3     \tab cube sum \cr
us4     \tab sum of 4th powers \cr
mean    \tab arithmetic mean \cr
css     \tab corrected square sum \cr
var     \tab variance \cr
stddev  \tab standard deviation \cr
stderrm \tab std. error of mean \cr
t       \tab t test statistics \cr
skew    \tab skewness \cr
kurt    \tab kurtosis \cr
bimod   \tab bimodality \cr
cv      \tab coeff. of variation (pct.)\cr
l4      \tab Blest's L4 location \cr
bkurt   \tab Blest's kurtosis \cr
}

Quantiles reported:

\tabular{ll}{
min2    \tab 2nd smallest \cr
p1      \tab 1st percentile \cr
p1_25   \tab 1.25th percentile \cr
p2      \tab 2nd percentile \cr
p2_5    \tab 2.5th percentile \cr
p5      \tab 5th percentile \cr
d1      \tab 1st decile \cr
o1      \tab 1st octile \cr
d2      \tab 2nd decile \cr
q1      \tab lower quartile \cr
d3      \tab 3rd decile \cr
o3      \tab 3rd octile \cr
d4      \tab 4th decile \cr
median  \tab median \cr
d6      \tab 6th decile \cr
o5      \tab 5th octile \cr
d7      \tab 7th decile \cr
q3      \tab upper quartile \cr
d8      \tab 8th decile \cr
o7      \tab 7th octile \cr
d9      \tab 9th decile \cr
p95     \tab 95th percentile \cr
p97_5   \tab 97.5th percentile \cr
p98     \tab 98th percentile \cr
p98_75  \tab 98.75th percentile \cr
p99     \tab 99th percentile \cr
max2    \tab 2nd largest \cr
}

Other derived statistics reported:

\tabular{ll}{
trimean \tab Tukey's trimean \cr
range   \tab range \cr
qrange  \tab quartile range \cr
lnotch  \tab lower boxplot notch \cr
unotch  \tab upper boxplot notch \cr
qmcoeff \tab quartile-median coeff. of var. \cr
qcoeff  \tab pure quartile coeff. of var. \cr
mskew   \tab Pearson's median-skewness \cr
yskew   \tab Yule's skewness \cr
qskew   \tab Bowley's quartile skewness \cr
h10skew \tab Hinkley's 1st decile skewness \cr
h5skew  \tab Hinkley's 5th percentile skewness \cr
h1skew  \tab Hinkley's 1st percentile skewness \cr
mkurt   \tab Moors' kurtosis \cr
cskurt  \tab Crow-Siddiqui's kurtosis \cr
}

(All kurtosis variants are normalized to equal 0 for a normal distribution)

\code{jk.means(input.table=..., analyse=..., where=..., group.by=...)} 
reports the approximate value (v) and the approximate protection error (e) 
for the above univariate statistics,
for each variable specified in \code{analyse},
based on the set of observations in \code{input.table} that fulfil the optional \code{where} condition,
optionally grouped by the hierarchical grouping dimensions specified in \code{group.by}.

Confidence intervals for the true (sample) value of the statistics can be constructed from v and e:
[v- e,v+ e] is a 75\% confidence interval,
[v-2e,v+2e] is a 95\% confidence interval, and
[v-3e,v+3e] is a 99\% confidence interval.}
\value{Approximate results and approximate protection errors,
either as a linearized data frame or as a nested list representing a multi-dimentional table (mudit format)}
\references{\url{http://www.unece.org/stats/documents/ece/ces/ge.46/2005/wp.39.e.pdf}
\url{http://www.unece.org/stats/documents/ece/ces/ge.46/2005/wp.39.add.1.e.pdf}}
\keyword{univar}
\keyword{robust}
\author{Jobst Heitzig \email{heitzig-j@users.sourceforge.net}}
\arguments{\item{connection}{MySQL database connection object as returned by \code{dbConnect}.}
\item{input.table}{String naming the database table to analyse.}
\item{analyse}{List specifying the analysis columns and the protection quartiles,
each entry being of the format \code{list(name="column name", quartiles=c(min,q1,median,q3,max))},
where \code{min,q1,median,q3,max} are real numbers that should resemble the 
true (population) quantiles of the variable
(\code{min} might be \code{-Inf}, \code{max} might be \code{Inf}).
If in doubt, estimate q1, median, and q3 from the sample, apply some rounding 
and use \code{quartiles=(-Inf, rounded sample q1, rounded sample median, rounded sample q3, Inf)}}
\item{where}{String containing a where condition (an SQL boolean expression used to filter rows).
Default: \code{"TRUE"}}
\item{group.by}{List specifying the desired grouping in the format \code{list(dim,...,dim)},
each \code{dim} being of the format
\code{list(name="column name", label="...", groups=list(group, ..., group))}
and each \code{group} being of the format
\code{list(level=l, label="...", condition="sql boolean expression")},
where \code{l} is the no. of the hierarchical level, 1 being the outmost level,
and the where condition can contain the placeholder "\code{x}" in place of the column name.
Default: \code{list(list(name="all", label="all", groups=list(list(level=1, label="all", condition="TRUE"))))}}
\item{mu0}{Comparison value for Student's one-sample t test.
Default: \code{0}}
\item{alpha.notch}{Confidence level for boxplot notches.
Default: \code{.025}}
\item{return.format}{Format of returned result: \code{"df"} for a linearized data frame, 
\code{"mudit"} for a nested list representing a multi-dimentional table (mudit format).
Default: \code{"df"}}
\item{debug}{If \code{1}, a lot of debugging info is output. DON'T USE IN PRODUCTION SINCE IT MIGHT REVEAL CONFIDENTIAL DATA!
Default: \code{0}}
\item{secret}{A string only known to the data provider and equal for all interrelated data tables of this provider.
CONFIDENTIALITY RELIES ON THIS BEING SECRET TO USERS!}}
\examples{jk.means(connection=my.mysql.connection, input.table='person',
analyse=list(list(name='age',label='Age in years',quartiles=c(0,20,40,60,Inf))))}
